Attribute VB_Name = "Module1"
Option Compare Database
' Make tables for each of the document node's children.
Sub TestBuildManyAccessTablesFromXml()
    Dim dom_document As DOMDocument

    ' Open the XML document.
    Set dom_document = New DOMDocument
    dom_document.Load "C:\VB XML\Src\Ch10\Stuff.xml"

    ' Make the tables.
    BuildManyAccessTablesFromXml dom_document.documentElement, 40

    ' Close the XML document.
    Set dom_document = Nothing
End Sub
' Make tables for each of the document node's children.
Sub BuildManyAccessTablesFromXml(ByVal document_node As IXMLDOMNode, Optional ByVal min_field_length As Integer = 100)
    Dim table_node As IXMLDOMNode

    ' Make the tables.
    For Each table_node In document_node.childNodes
        BuildAccessTableFromXml table_node, min_field_length
    Next table_node
End Sub
' Make a table named after the XML document's root node
' with fields given by its children.
Sub TestBuildAccessTableFromXml()
    Dim dom_document As DOMDocument

    ' Open the XML document.
    Set dom_document = New DOMDocument
    dom_document.Load "C:\VB XML\Src\Ch10\Books.xml"

    ' Make the table using the root node.
    BuildAccessTableFromXml dom_document.documentElement, 40

    ' Close the XML document.
    Set dom_document = Nothing
End Sub
' Make a table named after the given XML node with fields
' given by its children. Fields are VARCHARs with lengths
' big enough to hold the data or a minimum of min_field_length.
' This routine requires a reference to the DAO object library.
Private Sub BuildAccessTableFromXml(ByVal table_node As IXMLDOMNode, Optional ByVal min_field_length As Integer = 100)
    Dim record_node As IXMLDOMNode
    Dim field_node As IXMLDOMNode
    Dim num_fields As Integer
    Dim field_num As Integer
    Dim field_length() As Integer
    Dim new_table As DAO.TableDef
    Dim new_field As DAO.Field
    Dim rs As DAO.Recordset

    ' See how many fields the table will have.
    Set record_node = table_node.childNodes(1)
    num_fields = record_node.childNodes.length
    ReDim field_length(0 To num_fields - 1)

    ' Initialize the minimum field lengths.
    For field_num = 0 To num_fields - 1
        field_length(field_num) = min_field_length
    Next field_num

    ' Find the maximum field lengths.
    For field_num = 0 To num_fields - 1
        Set field_node = record_node.childNodes(field_num)
        If field_length(field_num) < Len(field_node.Text) Then
            field_length(field_num) = Len(field_node.Text)
        End If
    Next field_num

    ' Prepare to create the table.
    Set new_table = CurrentDb.CreateTableDef(table_node.nodeName)

    ' Create the fields.
    For field_num = 0 To num_fields - 1
        Set field_node = record_node.childNodes(field_num)
        Set new_field = new_table.CreateField(field_node.nodeName, dbText, field_length(field_num))
        new_table.Fields.Append new_field
    Next field_num

    ' Create the table.
    CurrentDb.TableDefs.Append new_table

    ' Connect to the new table.
    Set rs = CurrentDb.OpenRecordset(table_node.nodeName, dbOpenTable, dbAppendOnly)

    ' Load the data into the table.
    For Each record_node In table_node.childNodes
        ' Load this record.
        rs.AddNew

        ' Load the fields.
        For field_num = 0 To num_fields - 1
            Set field_node = record_node.childNodes(field_num)
            rs.Fields(field_num).Value = field_node.Text
        Next field_num

        ' Save the new record.
        rs.Update
    Next record_node

    ' Close the Recordset.
    rs.Close
    Set rs = Nothing
End Sub
